"use strict";

var _index = _interopRequireDefault(require("./index"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function mockProperty(event, func) {
  try {
    Object.defineProperty(window.HTMLMediaElement.prototype, event, {
      get: function get(attr) {
        return attr;
      },
      set: function set(callback) {
        return (func || callback)();
      }
    });
  } catch (e) {} // Expected redefine property error.

}

function commonSetup() {
  ['oncanplaythrough', 'onerror', 'onended'].forEach(function (e) {
    return mockProperty(e);
  });
  var error = Error();
  error.name = 'NotAllowedError';
  Object.defineProperty(global.window.HTMLMediaElement.prototype, 'play', {
    configurable: true,
    get: function get() {
      return function () {
        return Promise.reject(error);
      };
    }
  });
  this.error = error;
  this.host = 'http://faketesting-audiosequence.org/';
  this.files = Array(4).slice(1).fill(this.host).map(function (f, i) {
    return "".concat(f).concat(i, ".mp3");
  });
  this.player = new _index["default"]({
    files: this.files
  });
}

describe('Testing module main functionalities and units.', function () {
  var _this = this;

  beforeEach(commonSetup.bind(this));
  test('Test Module default parameters.', function () {
    expect(_this.player.files).toEqual(_this.files);
    expect(_this.player.repeats).toEqual(1);
    expect(_this.player.repeatWhole).toEqual(true);
    expect(_this.player.repeatEach).toEqual(false);
    expect(_this.player.repeatForever).toEqual(false);
    expect(_this.player.repeatDelay).toEqual(0);
    expect(_this.player.reverseOrder).toEqual(false);
    expect(_this.player.shuffleOrder).toEqual(false);
    expect(_this.player.volume).toEqual(0.5);
    expect(_this.player.autoStart).toEqual(false);
  });
  it('Test loading audio file through loader', function () {
    expect.assertions(1);
    return _this.player.loader(_this.files[0]).then(function (element) {
      return expect(element.src).toEqual(_this.files[0]);
    });
  });
  it('Test adding multiple audio files are loaded', function () {
    expect.assertions(3);
    return _this.player.load().then(function (files) {
      expect(files.map(function (f) {
        return f.src;
      })).toEqual(_this.files);
      expect(_this.player.playlist).toEqual(files);
      expect(_this.player.files).toEqual(files.map(function (f) {
        return f.src.replace(window.origin, '');
      }));
    });
  });
  it('Test auto teardown after loading a second time', function () {
    expect.assertions(2);
    return _this.player.load().then(function (files) {
      var elementsLength = document.body.children.length;
      var playlistLength = _this.player.playlist.length;

      _this.player.load().then(function (secondFiles) {
        expect(_this.player.playlist.length).toEqual(playlistLength);
        expect(document.body.children.length).toEqual(elementsLength);
      });
    });
  });
  it('Test autoplay policy instructions overlay', function () {
    _this.player.autoStart = true;
    expect.assertions(1);
    return _this.player.load().then(function (files) {
      return expect(document.getElementById('OverLayAutoPlay')).toBeTruthy();
    });
  });
  it('Test adding a new file promise', function () {
    var file = 'http://test.com/1.mp3';
    expect.assertions(1);
    return _this.player.add(file).then(function (element) {
      return expect(element.src).toEqual(file);
    });
  });
  it('Test adding already existing file rejection', function () {
    var file = 'http://test.com/1.mp3';
    expect.assertions(1);
    return _this.player.add(file).then(function (e) {
      return _this.player.add(file)["catch"](function (error) {
        return expect(error.message).toEqual('File already exists.');
      });
    });
  });
});