"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AudioSequence = void 0;

var _types = require("@babel/types");

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var AudioSequence = /*#__PURE__*/function () {
  /**
   * Utility to ease the process of plying audio elements in sequences.
   * @param {object} options contains the module options.
   */
  function AudioSequence() {
    var _this = this;

    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, AudioSequence);

    var AUTOPLAY_MSG = 'AutoPlay permission is lacking. Enable it then reload:';

    var b = function b(value, defValue) {
      return value === undefined ? defValue : !!value;
    };

    this.files = options.files || []; // files inserted will be stored in

    this.repeats = options.repeats || 1; // number of repeats to obey with some adjustments later

    this.repeatWhole = b(options.repeat_whole, true); // repeat all files as whole

    this.repeatEach = b(options.repeat_each, false); // repeat each file for the number of repeats

    this.repeatForever = b(options.repeat_forever, false); // to keep repeating endlessly

    this.repeatDelay = options.repeat_delay * 1000 || 0; // to add a time delay between each repeat

    this.reverseOrder = b(options.reverse_order, false); // to reverse the order list of audio files

    this.shuffleOrder = b(options.shuffle_order, false); // to randomly shuffle the order of the files list

    this.volume = options.volume || 0.5; // to set the default volume

    this.autoStart = b(options.auto_start, false); // to auto load and start playing as the module loads

    this.autoplayWarning = b(options.autoplay_warning, true); // to display warning if AutoPlay's disabled

    this.autoplayMessage = options.autoplay_message || AUTOPLAY_MSG; // message to show if AutoPlay's disabled

    this.playlist = []; // stack of audio elements playing

    this.current = 0; // index of the currently playing

    this.errors = []; // stack of errors encountered in loading

    this.loading = false; // indication of all files loaded

    this.repeatCounter = 0; // whole repeats index counter

    this.prePromises = []; // to resolve prior to ending transition

    this.postPromises = []; // to resolve after ending transition

    this.waiting = []; // array of files array to play after ending

    this.afterEffects = []; // array of functions to call after ending

    this.logger = undefined; // store the logging interval

    this.hasFiles = function () {
      return !!_this.playlist.length;
    };

    this.hasErrors = function () {
      return !!_this.errors.length;
    };

    this.getCurrent = function () {
      return {
        index: _this.current,
        item: _this.playlist[_this.current]
      };
    };

    this.getNext = function (p) {
      return _this.keepWithin(p ? _this.current - 1 : _this.current + 1, _this.playlist);
    };

    this.isMuted = function () {
      return _this.hasFiles() && _this.getCurrent().item.volume === 0;
    };

    this.isPaused = function () {
      return _this.hasFiles() && _this.getCurrent().item.paused;
    };

    this.notStarted = function () {
      return !_this.isPaused() && !_this.isActive();
    };

    this.getPlace = function (ele) {
      return _this.files.map(function (f) {
        return f.replace(window.origin, '');
      }).indexOf(ele.src.replace(window.origin, ''));
    };

    this.isLast = function () {
      return _this.playlist.length - 1 === _this.current;
    };

    this.isFinal = function () {
      var lastTrack = _this.playlist.length - 1 === _this.current;

      var lastRepeat = _this.getCurrent().item.repeats >= _this.repeats;

      return lastTrack && (_this.repeatEach ? lastRepeat : _this.repeatCounter >= _this.repeats);
    };

    this.isActive = function (item) {
      if (_this.hasFiles()) {
        item = item || _this.getCurrent().item;
        return item && !item.ended && item.currentTime > 0;
      } else return false;
    };

    this.mixIns = ['utils', 'constants', 'fetcher', 'controller', 'repeater', 'logger'];
    this.mixIns.forEach(function (mixin) {
      return Object.assign(_this, require("./mixins/".concat(mixin)));
    }); // if auto-play is disabled, will prompt the user with instructions.

    this.handleAutoPlayNotAllowed(); // auto start playing audio, or wait for DOM to load.

    if (this.autoStart) this.waitForDOM(this.load.bind(this));
  }

  _createClass(AudioSequence, [{
    key: "handleEnding",
    value: function handleEnding(event) {
      var _this2 = this;

      return new Promise(function (resolve) {
        var element = _this2.playlist.find(function (ele) {
          return ele.src === event.target.src;
        });

        var eleIndex = _this2.playlist.indexOf(element);

        var _ref = _this2.shuffleOrder ? _this2.choice(_this2.playlist) : _this2.reverseOrder ? _this2.keepWithin(eleIndex - 1, _this2.playlist) : _this2.keepWithin(eleIndex + 1, _this2.playlist),
            nextIndex = _ref.nextIndex,
            nextItem = _ref.nextItem;

        var zeroAndPlay = function zeroAndPlay(element) {
          element.volume = _this2.volume;
          element.currentTime = 0;
          var promise = (_this2.repeatEach || _this2.repeatWhole) && _this2.repeatDelay ? new Promise(function (resolve) {
            return setTimeout(function () {
              return resolve(element.play());
            }, _this2.repeatDelay);
          }) : element.play();
          return promise && promise.then ? promise.then(function () {
            return resolve(element);
          }) : resolve(element);
        };

        var commonNext = function commonNext() {
          if (nextItem) {
            _this2.current = nextIndex;
            nextItem.repeats = 1;
            return zeroAndPlay(nextItem);
          } else return resolve(element);
        };

        if (_this2.repeatEach) {
          var repeats = element.repeats || 1;

          if (repeats >= _this2.repeats && !_this2.repeatForever) {
            element.repeats = 0;
            return _this2.isLast() ? resolve(element) : commonNext();
          } else {
            element.repeats = repeats + 1;
            return zeroAndPlay(element);
          }
        } else if (_this2.repeatWhole) {
          if (_this2.repeatForever) return commonNext();else {
            if (_this2.repeatCounter >= _this2.repeats) return resolve(element);else {
              if (_this2.current === _this2.playlist.length - 2) _this2.repeatCounter += 1;
              return commonNext();
            }
          }
        } else {
          if (_this2.repeatForever || _this2.playlist.length - 1 >= _this2.current) return commonNext();else {
            _this2.current = 0;
            return resolve(element);
          }
        }
      });
    }
  }, {
    key: "tearDown",
    value: function tearDown() {
      if (this.isActive() && !this.isPaused()) this.getCurrent().item.pause();
      this.playlist.forEach(function (e) {
        return document.body.removeChild(e);
      });
      this.playlist = [];
      this.current = 0;
      this.repeatCounter = 0;
      this.errors = [];
      this.prePromises = [];
      this.postPromises = [];
    }
  }, {
    key: "load",
    value: function load() {
      var _this3 = this;

      if (this.hasFiles()) this.tearDown();
      return new Promise(function (resolve) {
        _this3.fetchAll().then(function (stack) {
          if (stack.length) {
            // keep original order
            _this3.playlist = Array(stack.length).fill();
            stack.forEach(function (ele) {
              _this3.playlist[_this3.getPlace(ele)] = ele;
            });
            _this3.playlist = _this3.playlist.filter(function (ele) {
              return !!ele;
            });
          }

          if (_this3.hasErrors()) _this3.errors.forEach(function (error) {
            return console.warn(error);
          });
          if (_this3.autoStart && _this3.hasFiles()) _this3.handlePlay(_this3.getCurrent().item.play());
          resolve(stack);
        });
      });
    }
  }]);

  return AudioSequence;
}();

exports.AudioSequence = AudioSequence;