"use strict";

module.exports = {
  play: function play() {
    var file = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

    if (file) {
      if (!this.hasFiles()) return false;
      var element = this.findFile(file);
      var index = this.playlist.indexOf(element);

      if (element) {
        this.pause(true);
        this.current = index;
        this.handlePlay(element.play());
        return true;
      } else return false;
    } else {
      var canPlay = this.hasFiles() && (!this.isActive() || this.isPaused());
      if (canPlay) this.handlePlay(this.getCurrent().item.play());
      return canPlay;
    }
  },
  playAfter: function playAfter() {
    var _this = this;

    var files = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
    if (this.isActive()) this.waiting.push(files);else {
      this.files = files;
      this.load().then(function () {
        return !_this.autoStart && _this.play();
      });
    }
  },
  doAfter: function doAfter(todo) {
    if (todo) this.afterEffects.push(todo);
  },
  replay: function replay() {
    var canReplay = this.isActive() || this.isPaused();

    if (canReplay) {
      var _this$getCurrent = this.getCurrent(),
          item = _this$getCurrent.item;

      item.currentTime = 0;
      if (this.isPaused()) this.handlePlay(item.play());
    }

    return canReplay;
  },
  pause: function pause() {
    var softStop = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
    var canPause = this.isActive() && !this.isPaused();

    if (canPause) {
      var _this$getCurrent2 = this.getCurrent(),
          item = _this$getCurrent2.item;

      item.pause();
      if (softStop) item.currentTime = 0;
    }

    return canPause;
  },
  stop: function stop() {
    var canStop = this.isActive();

    if (canStop) {
      var _this$getCurrent3 = this.getCurrent(),
          item = _this$getCurrent3.item;

      this.pause(true);
      item.currentTime = 0;
      this.current = 0;
    }

    return canStop;
  },
  next: function next(item) {
    var canMove = this.isActive(item) && this.hasFiles();

    if (canMove) {
      if (!item) item = this.getCurrent().item;
      item.currentTime = item.duration;
    }

    return canMove;
  },
  previous: function previous() {
    var canMove = this.isActive() && this.hasFiles();

    if (canMove) {
      var _this$getCurrent4 = this.getCurrent(),
          item = _this$getCurrent4.item;

      var _this$getNext = this.getNext(true),
          nextIndex = _this$getNext.nextIndex,
          nextItem = _this$getNext.nextItem;

      item.currentTime = 0;
      item.pause();
      this.current = nextIndex;
      this.handlePlay(nextItem.play());
    }

    return canMove;
  },
  forward: function forward() {
    var seconds = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
    var canForward = this.isActive() || this.isPaused();

    if (canForward) {
      var _this$getCurrent5 = this.getCurrent(),
          item = _this$getCurrent5.item;

      item.currentTime += seconds || item.duration / 6;
    }

    return canForward;
  },
  backward: function backward() {
    var seconds = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
    var canBackward = this.isActive() || this.isPaused();

    if (canBackward) {
      var _this$getCurrent6 = this.getCurrent(),
          item = _this$getCurrent6.item;

      item.currentTime -= seconds || item.duration / 6;
    }

    return canBackward;
  },
  mute: function mute() {
    var notMuted = !this.isMuted() && this.hasFiles();
    if (notMuted) this.playlist.forEach(function (ele) {
      ele.volume = 0;
    });
    return notMuted;
  },
  unmute: function unmute() {
    var volume = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0.5;
    this.playlist.forEach(function (ele) {
      ele.volume = volume;
    });
    return this.hasFiles();
  }
};