"use strict";

module.exports = {
  loader: function loader() {
    var _this = this;

    var file = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
    return new Promise(function (resolve, reject) {
      var element = document.createElement('audio');

      element.oncanplaythrough = function () {
        return resolve(element);
      };

      element.onerror = function (e) {
        var error = "Failed to load ".concat(file, ". ").concat(e);

        _this.errors.push(error);

        return reject(error);
      };

      element.onended = function (event) {
        _this.loading = true;
        return Promise.all(_this.prePromises).then(function (values) {
          return _this.handleEnding(event);
        }).then(function (ele) {
          return Promise.all(_this.postPromises);
        }).then(function (values) {
          _this.prePromises = [];
          _this.postPromises = [];
          _this.loading = false;

          if (!_this.isActive() && _this.isFinal() && element.src === _this.playlist.slice(-1)[0].src) {
            var waitingFiles = _this.waiting.shift();

            if (waitingFiles) {
              _this.files = waitingFiles;

              _this.load().then(function () {
                return !_this.autoStart && _this.play();
              });
            } else _this.afterEffects.forEach(function (f) {
              return f();
            });
          }
        })["catch"](function (e) {
          return console.warn(e);
        });
      };

      element.preload = 'auto';
      element.display = 'none';
      element.controls = false;
      element.volume = _this.volume;
      element.src = file;

      _this.waitForDOM(function () {
        return document.body.appendChild(element);
      });
    });
  },
  handlePlay: function handlePlay(promise) {
    var _this2 = this;

    if (promise) {
      promise["catch"](function (error) {
        return !_this2.isAutoPlayEnabled(error) && _this2.createOverlayInstructions();
      });
      return promise.then && this.prePromises.push(promise);
    }
  },
  add: function add() {
    var _this3 = this;

    var file = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
    return new Promise(function (resolve, reject) {
      var exists = !!_this3.playlist.find(function (e) {
        return e.src === file;
      });

      if (!exists) {
        _this3.loader(file).then(function (element) {
          _this3.playlist.push(element);

          if (_this3.autoStart) {
            _this3.pause(true);

            _this3.play(file);
          }

          resolve(element);
        })["catch"](function (error) {
          return reject(error);
        });
      } else reject(Error('File already exists.'));
    });
  },
  remove: function remove() {
    var file = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
    var element = this.playlist.find(function (e) {
      return e.src === file;
    });

    function tearDown(resolve) {
      this.playlist = this.playlist.filter(function (e) {
        return e.src !== element.src;
      });
      document.body.removeChild(element);
      if (resolve) return resolve('teared down');
    }

    if (this.isActive(element)) {
      this.addPromise(tearDown.bind(this));
      this.next(element);
    } else tearDown.bind(this)();

    return !!element;
  },
  fetchAll: function fetchAll() {
    var _this4 = this;

    this.loading = true;
    return new Promise(function (resolve) {
      Promise.all(_this4.files.map(function (file) {
        return _this4.loader(file);
      })).then(function (stack) {
        _this4.loading = false;
        resolve(stack);
      });
    });
  }
};